<?php
/**
 * Extend Media Upload
 *
 * @package    Extend Media Upload
 * @subpackage ExtendMediaUpload Main Functions
/*
	Copyright (c) 2018- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** ==================================================
 * Main Functions
 */
class ExtendMediaUpload {

	/** ==================================================
	 * Path
	 *
	 * @var $upload_dir  upload_dir.
	 */
	private $upload_dir;

	/** ==================================================
	 * Path
	 *
	 * @var $upload_path  upload_path.
	 */
	private $upload_path;

	/** ==================================================
	 * Construct
	 *
	 * @since 1.00
	 */
	public function __construct() {

		list( $this->upload_dir, $upload_url, $this->upload_path ) = $this->upload_dir_url_path();

		/* Original hook */
		add_action( 'emu_dir_select_box', array( $this, 'dir_select_box' ), 10, 1 );

		add_filter( 'upload_dir', array( $this, 'per_user_upload_dir' ) );
		add_action( 'wp_generate_attachment_metadata', array( $this, 'generate_metadata' ), 10, 2 );

	}

	/** ==================================================
	 * Scan directory
	 *
	 * @param string $dir  dir.
	 * @return array $dirlist
	 * @since 1.00
	 */
	private function scan_dir( $dir ) {

		$excludedir = '/^(?!.*(media-from-ftp-tmp|bulk-media-register-tmp'; /* tmp dir for Media from FTP and Bulk Media Register */
		global $blog_id;
		if ( is_multisite() && is_main_site( $blog_id ) ) {
			$excludedir .= '|\/sites\/';
		}
		$excludedir .= ')).*$/';

		$iterator = new RecursiveDirectoryIterator(
			$dir,
			FilesystemIterator::CURRENT_AS_FILEINFO |
			FilesystemIterator::KEY_AS_PATHNAME |
			FilesystemIterator::SKIP_DOTS
		);
		$iterator = new RecursiveIteratorIterator(
			$iterator,
			RecursiveIteratorIterator::SELF_FIRST
		);

		$iterator = new RegexIterator( $iterator, $excludedir, RecursiveRegexIterator::MATCH );

		$list = array();
		if ( ! empty( $iterator ) ) {
			foreach ( $iterator as $fileinfo ) {
				if ( $fileinfo->isDir() ) {
					$list[] = $fileinfo->getPathname();
				}
			}
		}

		asort( $list );
		return $list;

	}

	/** ==================================================
	 * Directory select box
	 *
	 * @param string $searchdir  searchdir.
	 * @since 1.00
	 */
	public function dir_select_box( $searchdir ) {

		if ( empty( $searchdir ) ) {
			$searchdir = '/';
		}

		$dirs = $this->scan_dir( $this->upload_dir );

		?>
		<select name="searchdir" style="width: 250px">
		<?php
		foreach ( $dirs as $linkdir ) {
			$linkdir = wp_normalize_path( $linkdir );
			$linkdirenc = $this->mb_utf8( str_replace( $this->upload_dir, '', $linkdir ) );
			$linkpath = str_replace( $this->upload_path, '', $linkdirenc );
			if ( $linkpath === $searchdir ) {
				?>
				<option value="<?php echo esc_attr( urlencode( $linkpath ) ); ?>" selected><?php echo esc_html( $linkpath ); ?></option>
				<?php
			} else {
				?>
				<option value="<?php echo esc_attr( urlencode( $linkpath ) ); ?>"><?php echo esc_html( $linkpath ); ?></option>
				<?php
			}
		}
		if ( '/' === $searchdir ) {
			?>
			<option value="<?php echo esc_attr( urlencode( '/' ) ); ?>" selected>/</option>
			<?php
		} else {
			?>
			<option value="<?php echo esc_attr( urlencode( '/' ) ); ?>">/</option>
			<?php
		}
		?>
		</select>
		<?php

	}

	/** ==================================================
	 * Per user upload dirctory
	 *
	 * @param array $original  original.
	 * @return array $modified
	 * @since 1.00
	 */
	public function per_user_upload_dir( $original ) {

		if ( ! function_exists( 'wp_get_current_user' ) ) {
			include_once( ABSPATH . 'wp-includes/pluggable.php' );
		}

		$extendmediaupload_settings = get_user_option( 'extendmediaupload', get_current_user_id() );

		if ( $extendmediaupload_settings ) {
			$original['subdir'] = $extendmediaupload_settings['subdir'];
		}
		$original['path'] = $original['basedir'] . $original['subdir'];
		$original['url'] = $original['baseurl'] . $original['subdir'];
		$modified = $original;

		return $modified;

	}

	/** ==================================================
	 * Generate metadata
	 *
	 * @param array $metadata  metadata.
	 * @param int   $attach_id  ID.
	 * @return array $metadata  metadata.
	 * @since 2.04
	 */
	public function generate_metadata( $metadata, $attach_id ) {

		if ( ! function_exists( 'wp_get_current_user' ) ) {
			include_once( ABSPATH . 'wp-includes/pluggable.php' );
		}

		$extendmediaupload_settings = get_user_option( 'extendmediaupload', get_current_user_id() );

		/* Date Time Regist */
		$dateset = $extendmediaupload_settings['dateset'];
		if ( function_exists( 'wp_date' ) ) {
			$postdategmt = wp_date( 'Y-m-d H:i:s', null, new DateTimeZone( 'UTC' ) );
		} else {
			$postdategmt = date_i18n( 'Y-m-d H:i:s', false, true );
		}
		if ( 'exif' === $dateset ) {
			$datetime = $this->get_date_check( get_attached_file( $attach_id ) );
			if ( $datetime ) {
				$postdategmt = get_gmt_from_date( $datetime );
			}
		}
		if ( 'new' <> $dateset ) {
			if ( 'fixed' === $dateset ) {
				$postdategmt = get_gmt_from_date( $extendmediaupload_settings['datefixed'] );
			}
			$postdate = get_date_from_gmt( $postdategmt );
			$up_post = array(
				'ID' => $attach_id,
				'post_date' => $postdate,
				'post_date_gmt' => $postdategmt,
				'post_modified' => $postdate,
				'post_modified_gmt' => $postdategmt,
			);
			wp_update_post( $up_post );
		}

		/* for Organize Media Library by Folders */
		do_action( 'oml_folders_term_create' );

		return $metadata;

	}

	/** ==================================================
	 * Date check
	 *
	 * @param string $file  file.
	 * @return string or bool $date
	 * @since 1.00
	 */
	private function get_date_check( $file ) {

		$date = false;

		$exifdata = @exif_read_data( $file, FILE, true );
		if ( isset( $exifdata['EXIF']['DateTimeOriginal'] ) && ! empty( $exifdata['EXIF']['DateTimeOriginal'] ) ) {
			$shooting_date_time = $exifdata['EXIF']['DateTimeOriginal'];
			$shooting_date = str_replace( ':', '-', substr( $shooting_date_time, 0, 10 ) );
			$shooting_time = substr( $shooting_date_time, 10 );
			$date = $shooting_date . $shooting_time;
		}

		return $date;

	}

	/** ==================================================
	 * Real Url
	 *
	 * @param  string $base  base.
	 * @param  string $relationalpath relationalpath.
	 * @return string $realurl realurl.
	 * @since  1.00
	 */
	private function realurl( $base, $relationalpath ) {

		$parse = array(
			'scheme'   => null,
			'user'     => null,
			'pass'     => null,
			'host'     => null,
			'port'     => null,
			'query'    => null,
			'fragment' => null,
		);
		$parse = wp_parse_url( $base );

		if ( strpos( $parse['path'], '/', ( strlen( $parse['path'] ) - 1 ) ) !== false ) {
			$parse['path'] .= '.';
		}

		if ( preg_match( '#^https?://#', $relationalpath ) ) {
			return $relationalpath;
		} elseif ( preg_match( '#^/.*$#', $relationalpath ) ) {
			return $parse['scheme'] . '://' . $parse['host'] . $relationalpath;
		} else {
			$base_path = explode( '/', dirname( $parse['path'] ) );
			$rel_path  = explode( '/', $relationalpath );
			foreach ( $rel_path as $rel_dir_name ) {
				if ( '.' === $rel_dir_name ) {
					array_shift( $base_path );
					array_unshift( $base_path, '' );
				} elseif ( '..' === $rel_dir_name ) {
					array_pop( $base_path );
					if ( count( $base_path ) === 0 ) {
						$base_path = array( '' );
					}
				} else {
					array_push( $base_path, $rel_dir_name );
				}
			}
			$path = implode( '/', $base_path );
			return $parse['scheme'] . '://' . $parse['host'] . $path;
		}

	}

	/** ==================================================
	 * Upload Path
	 *
	 * @return array $upload_dir,$upload_url,$upload_path  uploadpath.
	 * @since 1.00
	 */
	public function upload_dir_url_path() {

		$wp_uploads = wp_upload_dir();

		$relation_path_true = strpos( $wp_uploads['baseurl'], '../' );
		if ( $relation_path_true > 0 ) {
			$relationalpath = substr( $wp_uploads['baseurl'], $relation_path_true );
			$basepath       = substr( $wp_uploads['baseurl'], 0, $relation_path_true );
			$upload_url     = $this->realurl( $basepath, $relationalpath );
			$upload_dir     = wp_normalize_path( realpath( $wp_uploads['basedir'] ) );
		} else {
			$upload_url = $wp_uploads['baseurl'];
			$upload_dir = wp_normalize_path( $wp_uploads['basedir'] );
		}

		if ( is_ssl() ) {
			$upload_url = str_replace( 'http:', 'https:', $upload_url );
		}

		if ( $relation_path_true > 0 ) {
			$upload_path = $relationalpath;
		} else {
			$upload_path = str_replace( site_url( '/' ), '', $upload_url );
		}

		$upload_dir  = untrailingslashit( $upload_dir );
		$upload_url  = untrailingslashit( $upload_url );
		$upload_path = untrailingslashit( $upload_path );

		return array( $upload_dir, $upload_url, $upload_path );

	}

	/** ==================================================
	 * Multibyte UTF-8
	 *
	 * @param string $str  str.
	 * @return string $str
	 * @since 1.00
	 */
	private function mb_utf8( $str ) {

		if ( function_exists( 'mb_convert_encoding' ) ) {
			$encoding = implode( ',', mb_list_encodings() );
			$str = mb_convert_encoding( $str, 'UTF-8', $encoding );
		}

		return $str;

	}

}


